@@ -399,7 +399,7 @@ class Blobstore2Connection(object):
         log_dir = kwargs.pop("log_dir", "")
         try:
             correctcreds = BlobStore2.initializecreds(username=username, password=password, log_dir=log_dir)
-            bs2 = BlobStore2(log_dir=log_dir)
+            bs2 = BlobStore2()
             if not correctcreds:
                 security_state = int(bs2.get_security_state())
                 if security_state == 3:
@@ -450,6 +450,9 @@ class Blobstore2Connection(object):
                 if isinstance(body, bytes):
                     body = body.decode("utf-8")
                 body = json.dumps(body)
+            elif isinstance(body, bytes):
+                headers["Content-Type"] = "application/octet-stream"
+                body = bytearray(body)
             else:
                 headers["Content-Type"] = "application/x-www-form-urlencoded"
                 body = urlencode(body)
@@ -495,12 +498,14 @@ class Blobstore2Connection(object):
 
         if body and len(body) > 0:
             if isinstance(body, bytearray):
-                str1 = bytearray(str1.encode("ASCII")) + body
+                str1 = bytearray(str1.encode("utf-8")) + body
             else:
+                #if isinstance(body, bytes):
+                #    body = body.decode("utf-8")
                 str1 += body
 
         if not isinstance(str1, bytearray):
-            str1 = bytearray(str1.encode("ASCII"))
+            str1 = bytearray(str1.encode("utf-8"))
 
         if LOGGER.isEnabledFor(logging.DEBUG):
             try:
